# sympy je paket za simbolicko izracunavanje
# najpoznatiji alat za simbolicko izracunavanje je Wolphram Mathematica
import sympy as smp
import math as m


# da bismo nasli koren iz sedam mozemo da pozovemo
# klasicni numericki koren, tj. dobicemo numericku
# aproksimaciju vrednosti korena
print(f"Koren iz 7 (numeric): {m.sqrt(7)}")

# sa druge strane, simbolicki alati popur sympy
# koren iz sedam posmatraju kao simbol, tj. konstantu
# sa stvarnom vrednosti korena cime se postize veca
# preciznost racunanja
print(f"Koren iz 7 (symbolic): ", smp.sqrt(7))

# da bismo se ubedili da je simbolicko racunanje preciznije
# od numerickog, mozemo jednostavno da kvadriramo koren iz 7
print("(Koren iz 7)^2 (numeric)", m.pow(m.sqrt(7), 2))
print("(Koren iz 7)^2 (symbolic)", smp.Pow(smp.sqrt(7), 2))

# jos jedna prednost simbolickog izracunavanja je to sto nema
# prekoracenja, na primer, mozemo da pokusamo da izracunamo 7^55
print("7^50", smp.Pow(7, 55))


# pored konstanti mozemo da radimo i sa simbolima, tj. nezavisno
# promenljivim velicinama kao sto bi ih tumacili i u matematici
x,y = smp.symbols("x y")
print(x, y)

# sa definisanim izrazima mozemo da kreiramo izraze
# izrazi nisu stringovi, vec je ovo stvarni objekat sacinjen od
# nezavisno promenljivih
expr = 2*(x**2) + 5*y
# mozemo i da prikazemo izraz
print(expr)

# izrazi nam omogucavaju da sa njima postupamo kao sa izrazima u matematici
expr *= y
print(expr)

# treba primetiti da se izrazi ne mnoze automatski. ako zelimo da se izraz izmnozi
# treba da pozovemo metod expand
print(smp.expand(expr))