import sympy as smp

# kreiramo simbole
x, y, z = smp.symbols("x y z")

# kreiramo izraze od simbola
expr = 2*x + 5
expr2 = x**2 -x - 6
expr3 = x**2 + y**2
expr4 = (x - 6)/(y + 5)

# prikazujemo nase izraze na citljiviji nacin
smp.pretty_print(expr)
smp.pretty_print(expr2)
smp.pretty_print(expr3)
smp.pretty_print(expr4)

# mozemo da odredimo korene nasih izraza
koreni = smp.solve(expr2) # odredjuje resenje jednacine f(x) = 0, pri cemu f(x) mora biti algebarski izraz
print(koreni)

# izraze mozemo da evaluiramo, tj. da racuamo njihove vrednosti
# nakon zadavanja vrednosti promenljivama
print(expr2.subs(x, koreni[0])) # zamenjujemo vrednost simbola x sa prvim korenom
print(expr2.subs(x, koreni[1])) # zamenjujemo vrednost simbola x sa drugim korenom

# u slucaju da izraz ima vise promenljivih mozemo da menjamo pojedinacne na sledeci nacin
print(expr3.subs(x, 5))
# ali mozemo da menjamo odjednom i vise promenljivih
# opcije su da koristimo listu n-torki ili recnika
print(expr3.subs([(x,2),(y,3)])) # lista ntorki
print(expr3.subs({x : 2, y : 3})) # recnik

# pored zamene konstanti, metod subs nam dozvoljava i da izvrsimo smenu promenljivih
t = smp.symbols("t")
print(expr3.subs({x: t**2, y: 3*smp.sqrt(t)}))

# pored zamena vrednosti promenljivih nad izrazima su definisane jos tri kljucne funkcije
# expand() - mnozi sve clanove izraza
# simplify() - uproscava izraz tako da se dobije najjednostavniji oblik zapisa (nije isto sto i faktorisanje)
# factor() - rastavlja izraz na faktore

# kreiramo novi pomocni izraz
expr = (x+1)*(x+5)*(x-6)
# ukoliko odstampamo ovaj izraz, videcemo da sympy nece automatski
# da izmnozi zagrade
smp.pretty_print(expr)
# ukoliko zelimo da izmnozimo zagrade moramo da koristimo metod expand
izmnozeni_expr = smp.expand(expr)
smp.pretty_print(izmnozeni_expr)

# bitno je razlikovati uproscavanje od faktorisanja izraza
# faktorisanjem se dobija izraz jednak polaznom
uprosceni_izraz = smp.simplify(izmnozeni_expr)
smp.pretty_print(uprosceni_izraz) # ove ne postoji jednostaniviji nacin zapisa

faktorisani_izraz = smp.factor(izmnozeni_expr)
smp.pretty_print(faktorisani_izraz)

# simplify se koristi u sledecim slucajevima
expr7 = x*(y+5)
expr8 = smp.expand(expr7)
smp.pretty_print(smp.expand(expr7)) # x*y + 5*y
smp.pretty_print(smp.simplify(expr8)) # x*(y+5)

