import sympy as smp

# simbolicko resavanje jednacina

# prvo kreiramo nekoliko simbola
x, y, z, n= smp.symbols("x y z n")

# jednacine u sympy definisu se pomocu Eq konstruktora
# neka je data jednacina x^2 == 5, definisemo je na sledeci nacin
# prvi argument je desna strana, drugi argument je leva strana jednacine
eq1 = smp.Eq(x**2, 5)
# stampamo jednacinu
smp.pretty_print(eq1)

# resavanje jednacine radi se pomocu metoda solveset().
# solve mozemo da koristimo da bismo resili algebarske jednacine
sol = smp.solveset(eq1)
# ukoliko nasa jednacina ima vise promenljivih moramo da navedemo po 
# kojoj promenljivoj resavamo jednacinu kao drugi argument
sol = smp.solveset(eq1, x)
# mozemo da prikazemo resenja
smp.pretty_print(sol)

# i da ih izvucemo. prvo kastujemo resenje u listu
resenja = list(sol)
# i onda redom mozemo da ih indeksiramo
print("Resenja jednacine: ", resenja[0], ",", resenja[1])

# drugi pristup je da koristimo direktno izraze 
# tada nasa jednacina mora biti oblika f(x) == 0
sol = smp.solveset(x**2 - 5, x)
smp.pretty_print(sol)


# pored solveset mozemo da koristimo i metod solve ako smo sigurni da nase resenje nije
# beskonacan skup
# razlika izmedju solve i solveset je u tome sto kada imamo periodicne funkcije, solve vraca
# samo prvo resenje, a solveset sva moguca resenja (cak i ako ih je beskonacno)
# da bismo koristili solve, izraz mora biti zadat u obliku f(x) == 0
sol = smp.solve(eq1, x)
print(sol)
sol = smp.solve(x**2 - 5, x)
print(sol)


# naredni primer je trigonometrijska jednacina
eq2 = smp.Eq(smp.cos(x) - smp.sin(x), 0)
smp.pretty_print(eq2)

# pokusacemo da je resimo na isti nacin kao i prethodnu
sol2 = smp.solveset(eq2, x)
# i da prikazemo resenje
smp.pretty_print(sol2)
# resenje u ovom slucaju je beskonacan skup

# ako nam treba konacno resenje, mozemo da pokusamo sa solve
# za razliku od solveset, solve ce nam dati prvo resenje, a ne sva
smp.pretty_print(smp.solve(eq2, x))

# nisu sve jednacine simbolicki resive
# resenje ove jednacine je moguce odrediti samo numericki
eq3 = smp.Eq(smp.cos(x), x)

# pokusacemo da je resimo
sol3 = smp.solveset(eq3, x)
# i da prikazemo resenje
smp.pretty_print(sol3)
# i videcemo da je resenje polazne jednacine zapravo resenje
# jednacine -x + cos(x) == 0

# tj. solveset, ne ume da je resi


# kako resiti sistem linearnih jednacina?
eq_l1 = smp.Eq(x, 3*y + z)
eq_l2 = smp.Eq(5*x - 3*z, y)

# imamo nepotpun sistem i zelimo da ga resimo
# koristimo metod linsolve koji kao argumente prihvata
# listu jednacina i listu promenljivih po kojima treba da ih resimo
sol_l = smp.linsolve([eq_l1, eq_l2], [x, y, z])
# prikazujemo resenje
smp.pretty_print(sol_l)
# i ako nam treba resenje, mozemo da zamenimo vrednost slobodnog parametra
smp.pretty_print(sol_l.subs(z,3))
